%option 8bit
%option noyywrap
%option nostdinit
%option yylineno
%option case-insensitive

%{
    #include <iostream>
    #include <string>
    using namespace std;
    int yylex();
    #include "parser.cpp"
%}


LETTER [a-zA-Z]
DIGIT [0-9]
%% // --------------------------------------------------------------------------
[\t\ \n]*                               {}
"//"[^\n]*                              {}
"/*"([^*]|"*"[^/])*"*"+"/"              {}
"("                                     {if(DEBUG_LEXER) cerr << "OPEN "; return OPEN;}
")"                                     {if(DEBUG_LEXER) cerr << "CLOSE ";return CLOSE;}
"["                                     {if(DEBUG_LEXER) cerr << "OPENBRACKET ";return OPENBRACKET;}
"]"                                     {if(DEBUG_LEXER) cerr << "CLOSEBRACKET ";return CLOSEBRACKET;}
"="                                     {if(DEBUG_LEXER) cerr << "EQ ";return EQ;}
";"                                     {if(DEBUG_LEXER) cerr << "SEMICOLON\n\n";return SEMICOLON;}
","                                     {if(DEBUG_LEXER) cerr << "COMMA ";return COMMA;}

"\^"                                    {if(DEBUG_LEXER) cerr << "POW ";return POW;}
"_"                                     {if(DEBUG_LEXER) cerr << "INDEX ";return INDEX;}
":arity"                                {if(DEBUG_LEXER) cerr << "ARITY ";return ARITY;}
"proj"                                  {if(DEBUG_LEXER) cerr << "PROJ ";return PROJ;}
"succ"                                  {if(DEBUG_LEXER) cerr << "SUCC ";return SUCC;}
"rec"                                   {if(DEBUG_LEXER) cerr << "REC ";return REC;}
"comp"                                  {if(DEBUG_LEXER) cerr << "COMP ";return COMP;}
"\\mu"|"\xce\xbc"                       {if(DEBUG_LEXER) cerr << "MU ";return MU;}

{LETTER}({LETTER}|{DIGIT})*             {if(DEBUG_LEXER) cerr << "ID (" << yytext << ") ";yylval.text = new string(yytext); return ID;}
{DIGIT}+                                {if(DEBUG_LEXER) cerr << "INT (" << yytext << ") ";int x = atoi(yytext); yylval.number = new (unsigned int)(x >= 0 ? x : -x); return INT;}

.|\n                                    {cerr << "\n\nLEXICAL ERROR"
                                              << ": Unexpected character \""
                                              << yytext[0] << "\"\n";}
%% //---------------------------------------------------------------------------
